@@ -13,7 +13,7 @@ our @EXPORT_OK = (
 );
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = "v0.25.0";
+our $VERSION = "v0.25.1";
 
 1;
 __END__
@@ -178,7 +178,7 @@ L<https://rxjs.dev/api/index/const/EMPTY>
 
 =item rx_fork_join
 
-L<https://rxjs.dev/api/index/const/forkJoin>
+L<https://rxjs.dev/api/index/function/forkJoin>
 
     # [30, 3, 'c']
     rx_fork_join([
